/// <reference types="react" />
export type VirtualOverflowVisibleRect = {
    top: number;
    left: number;
    bottom: number;
    right: number;
    contentOffsetTop: number;
    contentOffsetLeft: number;
    contentVisibleHeight: number;
    contentVisibleWidth: number;
};
export type VirtualOverflowCalcVisibleRectFn = (element: HTMLElement, frameTime: number) => VirtualOverflowVisibleRect;
export type VirtualOverflowRenderItem1DFn = (itemIndex: number, offsetPx: number) => React.ReactNode;
export type VirtualOverflowRenderItem2DFn = (itemIndexX: number, leftOffsetPx: number, itemIndexY: number, topOffsetPx: number) => React.ReactNode;
export type UseCalcVirtualOverflowParams = {
    containerRef: React.MutableRefObject<HTMLElement>;
    itemsLengthX?: number;
    itemsLengthY?: number;
    /** if undefined, then horizontal calculation will be skipped */
    itemWidth?: number;
    /** if undefined, then vertical calculation will be skipped */
    itemHeight?: number;
    /** default=3 */
    overscanItemsCount?: number;
    calcVisibleRect?: VirtualOverflowCalcVisibleRectFn;
};
export type UseVirtualOverflowParamsGrid = {
    containerRef: React.MutableRefObject<HTMLElement>;
    itemsLengthX: number;
    itemsLengthY: number;
    itemWidth: number;
    itemHeight: number;
    overscanItemsCount?: number;
    renderItem: VirtualOverflowRenderItem2DFn;
    calcVisibleRect?: VirtualOverflowCalcVisibleRectFn;
};
export type UseVirtualOverflowParamsY = {
    containerRef: React.MutableRefObject<HTMLElement>;
    itemsLengthY: number;
    itemHeight: number;
    overscanItemsCount?: number;
    renderItem: VirtualOverflowRenderItem1DFn;
    calcVisibleRect?: VirtualOverflowCalcVisibleRectFn;
};
export type UseVirtualOverflowParamsX = {
    containerRef: React.MutableRefObject<HTMLElement>;
    itemsLengthX: number;
    itemWidth: number;
    overscanItemsCount?: number;
    renderItem: VirtualOverflowRenderItem1DFn;
    calcVisibleRect?: VirtualOverflowCalcVisibleRectFn;
};
export declare function virtualOverflowCalcVisibleRect(element: HTMLElement): {
    top: number;
    left: number;
    bottom: number;
    right: number;
    contentOffsetTop: number;
    contentOffsetLeft: number;
    contentVisibleHeight: number;
    contentVisibleWidth: number;
};
export declare function virtualOverflowCalcItems(contentOffsetStart: number, contentVisibleSize: number, itemSize: number, overscanItemsCount: number, itemsLength: number): {
    itemStart: number;
    itemLen: number;
};
export declare function useCalcVirtualOverflow(params: UseCalcVirtualOverflowParams, deps: any[]): {
    itemSlice: {
        topStartIndex: number;
        lengthY: number;
        leftStartIndex: number;
        lengthX: number;
    };
    updateViewRect: () => void;
};
export declare function useVirtualOverflowY(params: UseVirtualOverflowParamsY, deps?: any[]): {
    renderedItems: import("react").ReactNode[];
    updateViewRect: () => void;
    itemSlice: {
        topStartIndex: number;
        lengthY: number;
        leftStartIndex: number;
        lengthX: number;
    };
};
export declare function useVirtualOverflowX(params: UseVirtualOverflowParamsX, deps?: any[]): {
    renderedItems: import("react").ReactNode[];
    updateViewRect: () => void;
    itemSlice: {
        topStartIndex: number;
        lengthY: number;
        leftStartIndex: number;
        lengthX: number;
    };
};
export declare function useVirtualOverflowGrid(params: UseVirtualOverflowParamsGrid, deps?: any[]): {
    renderedItems: import("react").ReactNode[];
    updateViewRect: () => void;
    itemSlice: {
        topStartIndex: number;
        lengthY: number;
        leftStartIndex: number;
        lengthX: number;
    };
};
